<?php
/**
 * @package     Joomla.Site
 * @subpackage  com_joomla_lms
 *
 */

namespace LMS\Libraries\Exception;

defined('_JEXEC') or die;

use LMS\Libraries\Message;
use LMS\Libraries\Exception;
use LMS\Libraries\Logger;
use LMS\Libraries\Routing;
use LMS\Libraries\ServiceLocator;

/**
 * @package     Joomla.Site
 * @subpackage  com_joomla_lms
 * @since       3.1
 */
class Handler
{
	/**
	 * @var Message\Manager
	 */
	private $msgManager;

	/**
	 * @var Logger
	 */
	private $logger;

	/**
	 * @var Routing\Handler
	 */
	private $router;

	public function __construct(Message\Manager $msgManager, Logger $logger, Routing\Handler $router)
	{
		$this->msgManager = $msgManager;
		$this->logger = $logger;
		$this->router = $router;
	}

	/**
	 * Method to handle given exception
	 *
	 * @param \Exception $e
	 */
	public function handle(\Exception $e)
	{
		$this->logger->logError('Exception occurred: ' . $e->getMessage());

		$handlerMethod = 'handle' . get_class($e);
		$handlerMethod = str_replace('\\', '', $handlerMethod);

		if(DEVMODE == true){
			$this->msgManager->error($e->getMessage());
			$trace = explode('#', $e->getTraceAsString());
			$this->msgManager->warning('<p>' . implode('</p><p>', $trace) . '</p>');
		}

		if(method_exists($this, $handlerMethod)){
			$this->$handlerMethod($e);
		}else{
			$this->handleDefault($e);
		}

	}



	/**
	 * Method to handle any exception for which there is no other handlers
	 *
	 * @param \Exception $e
	 */
	private function handleDefault(\Exception $e)
	{
		$this->msgManager->error('Error occurred. Please contact administrator');
		$this->router->redirectBack();
	}

	/**
	 * Method to handle InvalidArgumentException.
	 * InvalidArgumentException occurs when Joomla can not find controller matched to input parameters
	 *
	 * @param \InvalidArgumentException $e
	 */
	private function handleInvalidArgumentException(\InvalidArgumentException $e)
	{
		//TODO: add logging
	//	$this->msgManager->error($e->getMessage()); //TODO : replace message with Joomla language constant
		$this->router->redirectBack();
	}

	/**
	 * Method to handle Exception\UndefinedTool.
	 * Exception\UndefinedTool occurs when parameters for running specific tool has not been provided
	 * @param Exception\UndefinedTool $e
	 */
	private function handleLMSLibrariesExceptionUndefinedTool(Exception\UndefinedTool $e)
	{
		//TODO: add logging
	}

	/**
	 * Method to handle LMSLibrariesExceptionDatabaseError.
	 * Exception\Database\Error occurs when something wrong with query to database
	 * @param Exception\Database\Error $e
	 */
	private function handleLMSLibrariesExceptionDatabaseError(Exception\Database\Error $e)
	{
		$this->msgManager->error('Database error occurred. Please contact administrator');
		$this->router->redirectBack();
	}

	/**
	 * Method to handle LMSLibrariesExceptionHttpFailedrequest.
	 * Exception\Http\FailedRequest occurs when something wrong with sending request to remote services
	 * @param Exception\Http\FailedRequest $e
	 */
	private function handleLMSLibrariesExceptionHttpFailedrequest(Exception\Http\FailedRequest $e)
	{
		$this->msgManager->error('HTTP error occurred. Please contact administrator');
		$this->router->redirectBack();
	}

	/**
	 * Method to handle LMSLibrariesExceptionValidationError
	 * Exception\Validation\Error occurs when validation of data failed
	 * @param Exception\Validation\Error $e
	 */
	private function handleLMSLibrariesExceptionValidationError(Exception\Validation\Error $e)
	{
		$this->msgManager->notice('Validation error: ' . $e->getMessage());
		$redirectUrl = $e->getRedirectURL();
		if(empty($redirectUrl)){
			$this->router->redirectBack();
		}
		JLMSRedirect($e->getRedirectURL());
	}

	/**
	 * Method to handle LMSLibrariesExceptionOperationError.
	 * Exception\Operation\Error occurs when some errors during process appeared
	 * @param Exception\UndefinedTool $e
	 */
	private function handleLMSLibrariesExceptionOperationError(Exception\Operation\Error $e)
	{
		$this->msgManager->error($e->getMessage());
		$redirectUrl = $e->getRedirectURL();
		if(empty($redirectUrl)){
			$this->router->redirectBack();
		}
		JLMSRedirect($e->getRedirectURL());
	}

	/**
	 * Method to handle LMSLibrariesExceptionAccessForbidden.
	 * Exception\Access\Forbidden occurs when some errors during process appeared
	 * @param Exception\Access\Forbidden $e
	 */
	private function handleLMSLibrariesExceptionAccessForbidden(Exception\Access\Forbidden $e)
	{
		$this->msgManager->warning($e->getMessage());
		$this->router->redirectBack();
	}
}